@@ -1,24 +1,24 @@
 """
 Class OWTextableCooccurrence
-Copyright 2012-2017 LangTech Sarl (info@langtech.ch)
+Copyright 2012-2019 LangTech Sarl (info@langtech.ch)
 -----------------------------------------------------------------------------
-This file is part of the Orange-Textable package v3.0.
+This file is part of the Orange3-Textable package.
 
-Orange-Textable v3.0 is free software: you can redistribute it and/or modify
+Orange3-Textable is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
-Orange-Textable v3.0 is distributed in the hope that it will be useful,
+Orange3-Textable is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with Orange-Textable v3.0. If not, see <http://www.gnu.org/licenses/>.
+along with Orange3-Textable. If not, see <http://www.gnu.org/licenses/>.
 """
 
-__version__ = u'1.0.2'
+__version__ = u'1.0.4'
 __author__ = "Mahtab Mohammadi"
 __maintainer__ = "LangTech Sarl"
 
@@ -28,7 +28,7 @@ from LTTL.Table import IntPivotCrosstab
 from LTTL.Segmentation import Segmentation
 
 from .TextableUtils import (
-    OWTextableBaseWidget,
+    OWTextableBaseWidget, ProgressBar,
     InfoBox, SendButton, updateMultipleInputs, pluralize,
     SegmentationListContextHandler, SegmentationsInputList,
 )
@@ -400,9 +400,12 @@ class OWTextableCooccurrence(OWTextableBaseWidget):
         if units['annotation_key'] == u'(none)':
             units['annotation_key'] = None
 
+        self.controlArea.setDisabled(True)
+        self.infoBox.setText(u"Processing, please wait...", "warning")
+
         # Case1: sliding window
         if self.mode == 'Sliding window':
-            progressBar = gui.ProgressBar(
+            progressBar = ProgressBar(
                 self,
                 iterations=len(units['segmentation']) - (self.windowSize - 1)
             )
@@ -432,7 +435,7 @@ class OWTextableCooccurrence(OWTextableBaseWidget):
                 if units2['annotation_key'] == u'(none)':
                     units2['annotation_key'] = None
                 num_iterations = len(contexts['segmentation'])
-                progressBar = gui.ProgressBar(
+                progressBar = ProgressBar(
                     self,
                     iterations=num_iterations * 2
                 )
@@ -445,7 +448,7 @@ class OWTextableCooccurrence(OWTextableBaseWidget):
                 )
             else:
                 num_iterations = (len(contexts['segmentation']))
-                progressBar = gui.ProgressBar(
+                progressBar = ProgressBar(
                     self,
                     iterations=num_iterations
                 )
@@ -456,6 +459,9 @@ class OWTextableCooccurrence(OWTextableBaseWidget):
                     progress_callback=progressBar.advance,
                 )
             progressBar.finish()
+
+        self.controlArea.setDisabled(False)
+
         if len(table.row_ids) == 0:
             self.infoBox.setText(u'Resulting table is empty.', 'warning')
             self.send('Textable pivot crosstab', None)
@@ -525,6 +531,7 @@ class OWTextableCooccurrence(OWTextableBaseWidget):
             self.containingSegmentationBox.setVisible(False)
             self.slidingWindowBox.setVisible(True)
             self.sequenceLengthSpin.setDisabled(False)
+            self.intraSeqDelimLineEdit.setDisabled(False)
 
             self.windowSizeSpin.setRange(
                 2,
@@ -540,6 +547,8 @@ class OWTextableCooccurrence(OWTextableBaseWidget):
             elif len(self.segmentations) >= 2:
                 self.units2Box.setDisabled(False)
             self.slidingWindowBox.setVisible(False)
+            self.sequenceLengthSpin.setDisabled(self.coocWithUnits2)
+            self.intraSeqDelimLineEdit.setDisabled(self.coocWithUnits2)
             self.containingSegmentationBox.setVisible(True)
             self.contextSegmentationCombo.clear()
             try:
@@ -575,7 +584,6 @@ class OWTextableCooccurrence(OWTextableBaseWidget):
                     self.unit2AnnotationKey = u'(none)'
                 self.unit2AnnotationKey = self.unit2AnnotationKey
                 self.sequenceLength = 1
-                self.sequenceLengthSpin.setDisabled(True)
 
     def handleNewSignals(self):
         """Overridden: called after multiple singnals have been added"""
@@ -586,7 +594,7 @@ class OWTextableCooccurrence(OWTextableBaseWidget):
 
 if __name__ == '__main__':
     import sys
-    from PyQt4.QtGui import QApplication
+    from PyQt5.QtWidgets import QApplication
     import LTTL.Segmenter as Segmenter
     from LTTL.Input import Input
 
