@@ -1,24 +1,24 @@
 """
 Class OWTextableIntersect
-Copyright 2012-2017 LangTech Sarl (info@langtech.ch)
+Copyright 2012-2019 LangTech Sarl (info@langtech.ch)
 -----------------------------------------------------------------------------
-This file is part of the Orange-Textable package v3.0.
+This file is part of the Orange3-Textable package.
 
-Orange-Textable v3.0 is free software: you can redistribute it and/or modify
+Orange3-Textable is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
-Orange-Textable v3.0 is distributed in the hope that it will be useful,
+Orange3-Textable is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with Orange-Textable v3.0. If not, see <http://www.gnu.org/licenses/>.
+along with Orange3-Textable. If not, see <http://www.gnu.org/licenses/>.
 """
 
-__version__ = '0.15.1'
+__version__ = '0.15.2'
 
 
 import LTTL.Segmenter as Segmenter
@@ -27,7 +27,7 @@ from LTTL.Segmentation import Segmentation
 from .TextableUtils import (
     OWTextableBaseWidget, InfoBox, SendButton, AdvancedSettings,
     pluralize, updateMultipleInputs, SegmentationListContextHandler,
-    SegmentationsInputList
+    SegmentationsInputList, ProgressBar
 )
 
 from Orange.widgets import widget, gui, settings
@@ -355,7 +355,9 @@ class OWTextableIntersect(OWTextableBaseWidget):
             copyAnnotations = True
 
         # Perform filtering...
-        progressBar = gui.ProgressBar(
+        self.infoBox.setText(u"Processing, please wait...", "warning")
+        self.controlArea.setDisabled(True)
+        progressBar = ProgressBar(
             self,
             iterations=num_iterations
         )
@@ -371,6 +373,7 @@ class OWTextableIntersect(OWTextableBaseWidget):
             progress_callback=progressBar.advance,
         )
         progressBar.finish()
+        self.controlArea.setDisabled(False)
         message = u'%i segment@p sent to output.' % len(filtered_data)
         message = pluralize(message, len(filtered_data))
         self.infoBox.setText(message)
@@ -473,7 +476,7 @@ if __name__ == '__main__':
     import sys
     import re
 
-    from PyQt4.QtGui import QApplication
+    from PyQt5.QtWidgets import QApplication
     from LTTL.Input import Input
 
     appl = QApplication(sys.argv)
