@@ -1,24 +1,24 @@
 """
 Class OWTextableVariety
-Copyright 2012-2017 LangTech Sarl (info@langtech.ch)
+Copyright 2012-2019 LangTech Sarl (info@langtech.ch)
 -----------------------------------------------------------------------------
-This file is part of the Orange-Textable package v3.0.
+This file is part of the Orange3-Textable package.
 
-Orange-Textable v3.0 is free software: you can redistribute it and/or modify
+Orange3-Textable is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
-Orange-Textable v3.0 is distributed in the hope that it will be useful,
+Orange3-Textable is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with Orange-Textable v3.0. If not, see <http://www.gnu.org/licenses/>.
+along with Orange3-Textable. If not, see <http://www.gnu.org/licenses/>.
 """
 
-__version__ = '0.13.4'
+__version__ = '0.13.7'
 
 
 from LTTL.Table import Table
@@ -26,7 +26,7 @@ from LTTL.Segmentation import Segmentation
 import LTTL.Processor as Processor
 
 from .TextableUtils import (
-    OWTextableBaseWidget, SegmentationListContextHandler,
+    OWTextableBaseWidget, SegmentationListContextHandler, ProgressBar,
     SegmentationsInputList, InfoBox, SendButton, updateMultipleInputs
 )
 import Orange.data
@@ -34,7 +34,7 @@ from Orange.widgets import widget, gui, settings
 
 
 class OWTextableVariety(OWTextableBaseWidget):
-    """Orange widget for mesuring variety of text units"""
+    """Orange widget for measuring variety of text units"""
 
     name = "Variety"
     description = "Measure the variety of segments"
@@ -58,6 +58,7 @@ class OWTextableVariety(OWTextableBaseWidget):
     unitWeighting = settings.Setting(False)
     measurePerCategory = settings.Setting(False)
     categoryWeighting = settings.Setting(False)
+    adjustSubsampleSize = settings.Setting(True)
     applyResampling = settings.Setting(False)
     numSubsamples = settings.Setting(100)
     subsampleSize = settings.ContextSetting(50)
@@ -216,6 +217,19 @@ class OWTextableVariety(OWTextableBaseWidget):
                 u"rather than unweighted average)."
             ),
         )
+        gui.separator(widget=iBox, height=3)
+        gui.checkBox(
+            widget=iBox,
+            master=self,
+            value='adjustSubsampleSize',
+            label=u'Dynamically adjust subsample size',
+            callback=self.sendButton.settingsChanged,
+            tooltip=(
+                u"Attempt to make variety estimation more robust\n"
+                u"by using the RMSP subsample size adjustment method\n"
+                u"described in Xanthos and Guex 2015."
+            ),
+        )
         self.measurePerCategoryCheckbox.disables.append(iBox)
         if self.measurePerCategory:
             iBox.setDisabled(False)
@@ -348,9 +362,10 @@ class OWTextableVariety(OWTextableBaseWidget):
             callback=self.sendButton.settingsChanged,
             tooltip=(
                 u"Check this box if you want to compute the average\n"
-                u"variety per subsample."
+                u"(or expected) variety per subsample."
             ),
         )
+        # TODO: document RMSP!
         gui.separator(widget=self.resamplingBox, height=3)
         iBox2 = gui.indentedBox(
             widget=self.resamplingBox,
@@ -458,21 +473,28 @@ class OWTextableVariety(OWTextableBaseWidget):
         categories = {
             'annotation_key': self.categoryAnnotationKey or None,
             'weighting': self.categoryWeighting,
+            'adjust': self.adjustSubsampleSize,
         }
         if categories['annotation_key'] == u'(none)':
             categories['annotation_key'] = None
 
+        self.infoBox.setText(u"Processing, please wait...", "warning")
+        self.controlArea.setDisabled(True)
+
         # Case 1: sliding window...
         if self.mode == 'Sliding window':
 
-            num_iterations = len(units['segmentation']) - (self.windowSize - 1)
-            if self.applyResampling:
-                num_iterations += num_iterations * self.numSubsamples
+            num_rows = len(units['segmentation']) - (self.windowSize - 1)
+            if self.measurePerCategory:
+                if self.applyResampling:
+                    num_iterations = num_rows * 3
+                else:
+                    num_iterations = num_rows * 2
             else:
-                num_iterations *= 2
+                num_iterations = num_rows / 2
 
             # Measure...
-            progressBar = gui.ProgressBar(
+            progressBar = ProgressBar(
                 self,
                 iterations=num_iterations
             )
@@ -502,21 +524,22 @@ class OWTextableVariety(OWTextableBaseWidget):
                 if contexts['annotation_key'] == u'(none)':
                     contexts['annotation_key'] = None
                 num_contexts = len(contexts['segmentation'])
-                num_iterations = num_contexts
+                if self.measurePerCategory and self.applyResampling:
+                    num_iterations = num_contexts * 3
+                else:
+                    num_iterations = num_contexts
             # Parameters for mode 'No context'...
             else:
                 contexts = None
                 num_iterations = (
                     len(units['segmentation']) - (self.sequenceLength - 1)
                 )
-                num_contexts = 1
-            if self.applyResampling:
-                num_iterations += num_contexts * self.numSubsamples
-            else:
-                num_iterations += num_contexts
+                if self.measurePerCategory and self.applyResampling:
+                    num_iterations *= 2
+
 
             # Measure...
-            progressBar = gui.ProgressBar(
+            progressBar = ProgressBar(
                 self,
                 iterations=num_iterations
             )
@@ -532,6 +555,8 @@ class OWTextableVariety(OWTextableBaseWidget):
             )
             progressBar.finish()
 
+        self.controlArea.setDisabled(False)
+
         if not len(table.row_ids):
             self.infoBox.setText(u'Resulting table is empty.', 'warning')
             self.send('Textable table', None)
@@ -589,14 +614,14 @@ class OWTextableVariety(OWTextableBaseWidget):
                 len(self.segmentations[self.units][1])
             )
             self.sequenceLength = self.sequenceLength or 1
-            if self.sequenceLength > 1:
-                self.categoriesBox.setDisabled(True)
-            else:
-                self.categoriesBox.setDisabled(False)
-            if self.measurePerCategory:
-                self.sequenceLengthSpin.setDisabled(True)
-            else:
-                self.sequenceLengthSpin.setDisabled(False)
+            self.categoriesBox.setDisabled(self.sequenceLength > 1)
+            self.sequenceLengthSpin.setDisabled(self.measurePerCategory)
+            # if self.mode == u'Sliding window':
+            #     self.numSubsampleSpin.setDisabled(False)
+            # else:
+            self.numSubsampleSpin.setDisabled(
+                not(self.measurePerCategory or self.unitWeighting)
+            )
             self.contextsBox.setDisabled(False)
             self.subsampleSizeSpin.setRange(
                 1,
@@ -642,7 +667,7 @@ class OWTextableVariety(OWTextableBaseWidget):
 
 if __name__ == '__main__':
     import sys, re
-    from PyQt4.QtGui import  QApplication
+    from PyQt5.QtWidgets import QApplication
     import LTTL.Segmenter as Segmenter
     from LTTL.Input import Input
 
